//
//  NSLocale+RCExtensions.h
//  Purchases
//
//  Created by Jacob Eiting on 1/8/18.
//  Copyright © 2018 Purchases. All rights reserved.
//

#import <Foundation/Foundation.h>

@interface NSLocale (RCExtensions)

- (nullable NSString *)rc_currencyCode;

@end
