//
//  Purchases.h
//  Purchases
//
//  Created by Jacob Eiting on 9/27/17.
//  Copyright © 2017 RevenueCat, Inc. All rights reserved.
//

#import <StoreKit/StoreKit.h>

FOUNDATION_EXPORT double PurchasesVersionNumber;
FOUNDATION_EXPORT const unsigned char PurchasesVersionString[];

#import <Purchases/RCPurchases.h>
#import <Purchases/RCPurchaserInfo.h>
#import <Purchases/RCIntroEligibility.h>
