//
//  RCPurchaserInfo+React.h
//  RNPurchases
//
//  Created by Jacob Eiting on 2/9/18.
//  Copyright © 2018 Facebook. All rights reserved.
//

#import <Purchases/Purchases.h>

@interface RCPurchaserInfo (React)

- (NSDictionary *)dictionary;

@end
